@@ -169,6 +169,10 @@ def NotEqualsInteger(a: AST, b: AST):
     return Not(EqualsInteger(a, b))
 
 
+def Negate(a: AST):
+    return SubtractInteger(Integer(0), a)
+
+
 EqualsBool = Iff
 
 # List Utils
@@ -268,6 +272,37 @@ def FoldList(l: AST, f: AST, a: AST):
     )
 
 
+def RFoldList(l: AST, f: AST, a: AST):
+    """Right fold over a list l operator f: accumulator -> list_elem -> accumulator with initial value a"""
+    return Apply(
+        Lambda(
+            ["op"],
+            RecFun(
+                Lambda(
+                    ["fold", "xs", "a"],
+                    Ite(
+                        NullList(Var("xs")),
+                        Var("a"),
+                        Apply(
+                            Var("op"),
+                            Apply(
+                                Var("fold"),
+                                Var("fold"),
+                                TailList(Var("xs")),
+                                Var("a"),
+                            ),
+                            HeadList(Var("xs")),
+                        ),
+                    ),
+                ),
+            ),
+        ),
+        f,
+        l,
+        a,
+    )
+
+
 def IndexAccessList(l: AST, i: AST):
     return Apply(
         RecFun(
